module Data.Serialize.Ldsf.Lean (
      composeRequest
    , composeResponse
    , getRequest
    , getResponse
    ) where

import Data.ByteString (ByteString)
import Data.ByteString as B
import Data.Serialize.Get (Get)
import Data.Serialize.Put (Put)
import qualified Data.Serialize.Ldsf as Ldsf
import qualified Data.Serialize.Ldsf.Types as Ldsf
import Network.Lean.Lai (Request(..), Response(..))
import Network.Lean.Types (parseInterfaceCategory, renderInterfaceCategory,
                           parseResponseType, renderResponseType)

-- | Convert a request to its binary representation.
composeRequest :: Request internal -> ByteString
composeRequest request = Ldsf.compose composePayload
    where
        composePayload :: Ldsf.Version -> Put
        composePayload protocolVersion = do
            Ldsf.putText8 $ requestHostname request
            Ldsf.putUInt8 (renderInterfaceCategory $ requestCategory request)
            Ldsf.putUInt8 $ requestVersion request
            Ldsf.putUInt8 $ requestFunction request
            Ldsf.putValues $ requestArguments request

-- | Convert a response to its binary representation.
composeResponse :: Response internal -> ByteString
composeResponse response = Ldsf.compose composePayload
    where
        composePayload :: Ldsf.Version -> Put
        composePayload protocolVersion = do
            Ldsf.putUInt8 (renderResponseType $ responseType response)
            Ldsf.putValues (responsePayload response)

-- | Read the request.
getRequest :: (Ldsf.Version -> Request internal -> Get [Ldsf.Value]) ->
              Ldsf.Version ->
              Get (Request internal)
getRequest getRequestArguments ldsfVersion = do
    hostname <- Ldsf.getText8
    category <- Ldsf.getUInt8
    categoryParsed <- 
        case parseInterfaceCategory category of
             Nothing -> fail $ "Failed to parse category. Supplied value: " ++ show category
             Just c  -> pure c
    version <- Ldsf.getUInt8
    function <- Ldsf.getUInt8
    let request = Request
            { requestHostname = hostname
            , requestCategory = categoryParsed
            , requestVersion = version
            , requestFunction = function
            , requestArguments = []
            , requestInternal = Nothing
            }
    arguments <- getRequestArguments ldsfVersion request
    pure $ request { requestArguments = arguments}

-- | Read a response.
getResponse :: (Ldsf.Version -> Get [Ldsf.Value]) ->
               Ldsf.Version ->
               Get (Response internal)
getResponse getResponsePayload ldsfVersion = do
    responseType <- Ldsf.getUInt8
    parsedResponseType <-
        case parseResponseType responseType of
             Nothing -> fail $ "Failed to parse response type. Supplied value: " ++ show responseType
             Just rt -> pure rt
    payload <- getResponsePayload ldsfVersion
    pure Response
        { responseType = parsedResponseType
        , responsePayload = payload
        , responseInternal = Nothing
        }
