module Network.Lean.Client.Internal (
      ClientRequestState
    , ClientResponseState
    , handleResponse
    ) where

import Data.ByteString (ByteString)
import qualified Data.ByteString as B
import Data.Serialize.Get (Get, Result(..))
import Data.Serialize.Ldsf (Header)
import qualified Data.Serialize.Ldsf as Ldsf
import qualified Data.Serialize.Ldsf.Types as Ldsf
import qualified Data.Serialize.Ldsf.Lean.InterfaceFunction as IF
import Data.Serialize.Ldsf.Lean (getResponse)
import Network.Lean.Lai (Response(..))
import Network.Simple.TCP (Socket, SockAddr, recv)

data ClientRequestState = ClientRequestState
    deriving (Show)
data ClientResponseState = ClientResponseState
    deriving (Show)

handleResponse :: IF.GetResult ->
                  Socket -> IO (Maybe (Header, Response ClientResponseState))
handleResponse getResult socket = handle getResult socket Nothing Nothing
    where
        handle :: IF.GetResult ->
                  Socket -> Maybe ByteString ->
                  Maybe (ByteString -> Result (Ldsf.Header, Response ClientResponseState)) ->
                  IO (Maybe (Header, Response ClientResponseState))
        handle getResult socket leftovers continuation = do
            -- Maybe we have leftovers, if we don't then we need to receive more data.
            maybeData <- case leftovers of
                Nothing -> recv socket 1024
                Just l  -> pure $ Just l
            
            -- Check if the connection was closed.
            case maybeData of
                Nothing -> pure Nothing
                Just rdata  -> do
                    let parseResult =
                            case continuation of
                                Nothing   -> Ldsf.parseIncremental (getResponse (\_ -> getResult)) rdata
                                Just cont -> cont rdata
                    case parseResult of
                        Fail err bs -> pure Nothing
                        Partial cont -> handle getResult socket Nothing (Just cont)
                        Done (header, response) bs  -> do
                            if B.length bs > 0
                               then fail "B.length bs > 0" -- TODO: FIXME: well, shit! this shouldn't happen!
                               else pure $ Just (header, response)
